@@ -22,7 +22,8 @@
 #' if geo == FALSE.
 #'
 #' @param geo_units (list; list("epochs", "periods")) Which geological units to
-#' include in the geo timescale.
+#' include in the geo timescale. May be "periods", "epochs", "stages", "eons", 
+#' "eras", or a list of two of those units.
 #'
 #' @param node_age_bars (logical; TRUE) Plot time tree with node age bars?
 #'
@@ -46,7 +47,12 @@
 #'
 #' @param tip_labels (logical; TRUE) Plot tip labels?
 #'
-#' @param tip_labels_italics (logical; TRUE) Plot tip labels in italics?
+#' @param tip_labels_italics (logical; FALSE) Plot tip labels in italics?
+#'
+#' @param tip_labels_formatted (logical; FALSE) Do the tip labels contain 
+#' manually added formatting information? Will set parse = TRUE in geom_text()
+#' and associated functions to interpret formatting. See ?plotmath for more.
+#' Cannot be TRUE if tip_labels_italics = TRUE.  
 #'
 #' @param tip_labels_remove_underscore (logical; FALSE) Should underscores be
 #' replaced by spaces in tip labels?
@@ -137,6 +143,7 @@ plotTreeFull <- function(tree,
 
                          tip_labels,
                          tip_labels_italics,
+                         tip_labels_formatted,
                          tip_labels_remove_underscore,
                          tip_labels_color,
                          tip_labels_size,
@@ -158,7 +165,7 @@ plotTreeFull <- function(tree,
   # enforce argument matching
   if (!is.list(tree))
     stop("tree should be a list of lists of treedata objects")
-  if (class(tree[[1]][[1]]) != "treedata")
+  if (!methods::is(tree[[1]][[1]], "treedata"))
     stop("tree should be a list of lists of treedata objects")
   vars <- colnames(tree[[1]][[1]]@data)
   if (is.logical(timeline) == FALSE)
@@ -180,9 +187,13 @@ plotTreeFull <- function(tree,
     stop("tip_labels should be TRUE or FALSE")
   if (is.logical(tip_labels_italics) == FALSE)
     stop("tip_labels_italics should be TRUE or FALSE")
+  if (is.logical(tip_labels_formatted) == FALSE)
+    stop("tip_labels_formatted should be TRUE or FALSE")
+  if (tip_labels_italics == TRUE & tip_labels_formatted == TRUE) 
+    stop("tip_labels_italics and tip_labels_formatted may not both be TRUE")
   if (.isColor(tip_labels_color) == FALSE)
     stop("tip_labels_color should be a recognized color")
-  if (class(node_pp) != "logical")
+  if (!methods::is(node_pp,"logical"))
     stop("node_pp should be TRUE or FALSE")
   if (node_pp) {
     if (length(node_pp_color) > 2)
@@ -223,26 +234,30 @@ plotTreeFull <- function(tree,
   if (is.list(geo_units)) {
     if (length(geo_units) != 2)
       stop(
-        "geo_units should be 'epochs', 'periods' or a list of both:
+        "geo_units should be 'periods', 'epochs', 'stages', 'eons', 
+         'eras', or a list of two of those units, such as:
         list('epochs','periods')"
       )
-    if (geo_units[[1]] != "epochs" &
-        geo_units[[1]] != "periods")
+    if (geo_units[[1]] %in% 
+        c('periods', 'epochs', 'stages', 'eons', 'eras')  == FALSE)
       stop(
-        "geo_units should be 'epochs', 'periods' or a list of both:
+        "geo_units should be 'periods', 'epochs', 'stages', 'eons', 
+         'eras', or a list of two of those units, such as:
         list('epochs','periods')"
       )
-    if (geo_units[[2]] != "epochs" &
-        geo_units[[2]] != "periods")
+    if (geo_units[[2]] %in% 
+        c('periods', 'epochs', 'stages', 'eons', 'eras')  == FALSE)
       stop(
-        "geo_units should be 'epochs', 'periods' or a list of both:
+        "geo_units should be 'periods', 'epochs', 'stages', 'eons', 
+         'eras', or a list of two of those units, such as:
         list('epochs','periods')"
       )
   } else {
-    if (geo_units != "epochs" &
-        geo_units != "periods")
+    if (geo_units %in% 
+        c('periods', 'epochs', 'stages', 'eons', 'eras') == FALSE)
       stop(
-        "geo_units should be 'epochs', 'periods' or a list of both:
+        "geo_units should be 'periods', 'epochs', 'stages', 'eons', 
+         'eras', or a list of two of those units, such as:
         list('epochs','periods')"
       )
   }
@@ -333,6 +348,7 @@ plotTreeFull <- function(tree,
         in the treedata object."
       )
   }
+
   # get dimensions
   n_node <- treeio::Nnode(phy)
   tree_height <- max(phytools::nodeHeights(phy@phylo))
@@ -370,10 +386,9 @@ plotTreeFull <- function(tree,
         }
       })
       minmax <- t(matrix(unlist(pp$data$age_0.95_HPD), nrow = 2))
+      max_age <- max(minmax, na.rm = TRUE)
     }
 
-    max_age <- max(minmax, na.rm = TRUE)
-
     interval <- max_age / 5
     dx <- max_age %% interval
 
@@ -428,7 +443,7 @@ plotTreeFull <- function(tree,
     pp <- pp + ggplot2::scale_x_continuous(
       name = "Age (Ma)",
       expand = c(0, 0),
-      limits = c(-max(minmax, na.rm = TRUE), tree_height /
+      limits = c(-max_age, tree_height /
                    2),
       breaks = -rev(seq(0, max_age +
                           dx, interval)),
@@ -646,6 +661,36 @@ plotTreeFull <- function(tree,
           hjust = 0,
           parse = TRUE
         )
+    } else if (tip_labels_italics == TRUE) {
+      pp <-
+        pp + ggplot2::annotate(
+          "text",
+          x = sampled_ancs$x,
+          y = sampled_ancs$y,
+          label = seq_len(nrow(sampled_ancs)),
+          vjust = -.5,
+          size = tip_labels_size,
+          color = tip_labels_color
+        ) +
+        ggplot2::annotate(
+          "text",
+          x = rep(-max(
+            unlist(pp$data$age_0.95_HPD), na.rm = TRUE
+          ),
+          times = nrow(sampled_ancs)),
+          y = seq(
+            from = ntips - space_labels,
+            by = -space_labels,
+            length.out = nrow(sampled_ancs)
+          ),
+          label = paste0(seq_len(nrow(sampled_ancs)),
+                         ": ",
+                         sampled_ancs$label),
+          size = tip_labels_size,
+          color = tip_labels_color,
+          hjust = 0,
+          parse = TRUE
+        )
     } else {
       pp <-
         pp + ggplot2::annotate(
@@ -725,7 +770,7 @@ plotTreeFull <- function(tree,
     } else {
       pp$data$extant <- TRUE
     }
-    if (tip_labels_italics) {
+    if (tip_labels_italics == TRUE) {
       pp <- pp + ggtree::geom_tiplab(
         ggplot2::aes(
           subset = extant & isTip,
@@ -753,6 +798,32 @@ plotTreeFull <- function(tree,
             parse = TRUE
           )
       }
+    } else if (tip_labels_formatted == TRUE ) {
+      pp <- pp + ggtree::geom_tiplab(
+        ggplot2::aes(subset = extant & isTip,
+                     label = label),
+        size = tip_labels_size,
+        offset = tip_labels_offset,
+        color = tip_labels_color,
+        parse = TRUE
+      )
+      if (tip_age_bars == TRUE) {
+        new_tip_df <-
+          dplyr::left_join(tip_df,
+                           pp$data[, c("label", "node")],
+                           by = c("node_id" = "node"))
+        pp <-
+          pp + ggplot2::annotate(
+            "text",
+            x = -new_tip_df$min + tip_labels_offset,
+            y = new_tip_df$y,
+            label = new_tip_df$label,
+            hjust = 0,
+            color = tip_labels_color,
+            size = tip_labels_size,
+            parse = TRUE 
+          )
+      }
     } else {
       pp <- pp + ggtree::geom_tiplab(
         ggplot2::aes(subset = extant & isTip,
@@ -814,7 +885,7 @@ plotTreeFull <- function(tree,
     pp$data[, col_num] <-
       as.numeric(as.data.frame(pp$data)[, col_num])
     name <- .titleFormat(color_branch_by)
-    pp <- pp +
+    pp <- pp + 
       ggplot2::aes(color = as.data.frame(pp$data)[, col_num]) +
       ggplot2::scale_color_gradient(
         low = branch_color[1],
@@ -856,7 +927,6 @@ plotTreeFull <- function(tree,
       xlim_max <- range(pp$data$x)[2]
     }
 
-
     pp <- pp + ggtree::xlim(xlim_min, xlim_max)
 
   }
